@@ -12,7 +12,7 @@ module Capistrano
 
       def execute_requested_actions_with_help(config)
         if options[:tasks]
-          task_list(config)
+          task_list(config, options[:tasks])
         elsif options[:explain]
           explain_task(config, options[:explain])
         else
@@ -20,11 +20,19 @@ module Capistrano
         end
       end
 
-      def task_list(config) #:nodoc:
-        tasks = config.task_list(:all)
+      def task_list(config, pattern = true) #:nodoc:
+        tool_output = options[:tool]
+
+        if pattern.is_a?(String)
+          tasks = config.task_list(:all).select {|t| t.fully_qualified_name =~ /#{pattern}/}
+        end
+        if tasks.nil? || tasks.length == 0
+          warn "Pattern '#{pattern}' not found. Listing all tasks.\n\n" if !tool_output && !pattern.is_a?(TrueClass)
+          tasks = config.task_list(:all)
+        end
 
         if tasks.empty?
-          warn "There are no tasks available. Please specify a recipe file to load."
+          warn "There are no tasks available. Please specify a recipe file to load." unless tool_output
         else
           all_tasks_length = tasks.length
           if options[:verbose].to_i < 1
@@ -38,19 +46,25 @@ module Capistrano
           max_length = MIN_MAX_LEN if max_length < MIN_MAX_LEN
 
           tasks.each do |task|
-            puts "cap %-#{longest}s # %s" % [task.fully_qualified_name, task.brief_description(max_length)]
+            if tool_output
+              puts "cap #{task.fully_qualified_name}"
+            else
+              puts "cap %-#{longest}s # %s" % [task.fully_qualified_name, task.brief_description(max_length)]
+            end
           end
 
-          if all_tasks_length > tasks.length
+          unless tool_output
+            if all_tasks_length > tasks.length
+              puts
+              puts "Some tasks were not listed, either because they have no description,"
+              puts "or because they are only used internally by other tasks. To see all"
+              puts "tasks, type `#{File.basename($0)} -vT'."
+            end
+
             puts
-            puts "Some tasks were not listed, either because they have no description,"
-            puts "or because they are only used internally by other tasks. To see all"
-            puts "tasks, type `#{File.basename($0)} -Tv'."
+            puts "Extended help may be available for these tasks."
+            puts "Type `#{File.basename($0)} -e taskname' to view it."
           end
-
-          puts
-          puts "Extended help may be available for these tasks."
-          puts "Type `#{File.basename($0)} -e taskname' to view it."
         end
       end
 
@@ -97,8 +111,15 @@ module Capistrano
       end
 
       def output_columns #:nodoc:
-        @output_columns ||= self.class.ui.output_cols > 80 ? 80 : self.class.ui.output_cols
+        if ( @output_columns.nil? ) 
+          if ( self.class.ui.output_cols.nil? || self.class.ui.output_cols > 80 )
+            @output_columns = 80 
+          else
+            @output_columns = self.class.ui.output_cols
+          end
+        end
+        @output_columns 
       end
     end
   end
-end
\ No newline at end of file
+end
