@@ -24,6 +24,7 @@ module Capistrano
       # line switches for Capistrano, and what their corresponding behaviors
       # are.
       def option_parser #:nodoc:
+        @logger = Logger.new
         @option_parser ||= OptionParser.new do |opts|
           opts.banner = "Usage: #{File.basename($0)} [options] action ..."
 
@@ -43,7 +44,7 @@ module Capistrano
             "A recipe file to load. May be given more than once."
           ) { |value| options[:recipes] << value }
 
-          opts.on("-H", "--long-help", "Explain these options.") do
+          opts.on("-H", "--long-help", "Explain these options and environment variables.") do
             long_help
             exit
           end
@@ -53,6 +54,19 @@ module Capistrano
             exit
           end
 
+					opts.on("-l", "--logger [STDERR|STDOUT|file]", 
+						"Choose logger method. STDERR used by default."
+					) do |value| 
+						options[:output] = if value.nil? || value.upcase == 'STDERR'
+																 # Using default logger.
+																 nil
+															 elsif value.upcase == 'STDOUT'
+																 $stdout
+															 else
+																 value
+															 end
+					end
+
           opts.on("-n", "--dry-run",
             "Prints out commands without running them."
           ) { |value| options[:dry_run] = true }
@@ -167,6 +181,7 @@ module Capistrano
           %w(Capfile capfile).each do |file|
             if File.file?(file)
               options[:recipes] << file
+              @logger.info "Using recipes from #{File.join(current,file)}"
               return
             end
           end
