@@ -34,7 +34,7 @@ _cset(:revision)  { source.head }
 # =========================================================================
 
 _cset(:source)            { Capistrano::Deploy::SCM.new(scm, self) }
-_cset(:real_revision)     { source.local.query_revision(revision) { |cmd| with_env("LC_ALL", "C") { `#{cmd}` } } }
+_cset(:real_revision)     { source.local.query_revision(revision) { |cmd| with_env("LC_ALL", "C") { run_locally(cmd) } } }
 
 _cset(:strategy)          { Capistrano::Deploy::Strategy.new(deploy_via, self) }
 
@@ -42,6 +42,7 @@ _cset(:release_name)      { set :deploy_timestamped, true; Time.now.utc.strftime
 
 _cset :version_dir,       "releases"
 _cset :shared_dir,        "shared"
+_cset :shared_children,   %w(system log pids)
 _cset :current_dir,       "current"
 
 _cset(:releases_path)     { File.join(deploy_to, version_dir) }
@@ -49,9 +50,9 @@ _cset(:shared_path)       { File.join(deploy_to, shared_dir) }
 _cset(:current_path)      { File.join(deploy_to, current_dir) }
 _cset(:release_path)      { File.join(releases_path, release_name) }
 
-_cset(:releases)          { capture("ls -x #{releases_path}").split.sort }
+_cset(:releases)          { capture("ls -xt #{releases_path}").split.reverse }
 _cset(:current_release)   { File.join(releases_path, releases.last) }
-_cset(:previous_release)  { File.join(releases_path, releases[-2]) }
+_cset(:previous_release)  { releases.length > 1 ? File.join(releases_path, releases[-2]) : nil }
 
 _cset(:current_revision)  { capture("cat #{current_path}/REVISION").chomp }
 _cset(:latest_revision)   { capture("cat #{current_release}/REVISION").chomp }
@@ -89,6 +90,61 @@ ensure
   ENV[name] = saved
 end
 
+# logs the command then executes it locally.
+# returns the command output as a string
+def run_locally(cmd)
+  logger.trace "executing locally: #{cmd.inspect}" if logger
+  command_present?(cmd)
+  `#{cmd}`
+end
+
+# tests if the given command is present on the local system
+def command_present?(cmd)
+  executable = cmd.to_s.split(" ").first
+  unless system("which #{executable}")
+    logger.important "executable '#{executable}' not present or not in $PATH on the local system!"
+  end
+end
+
+# If a command is given, this will try to execute the given command, as
+# described below. Otherwise, it will return a string for use in embedding in
+# another command, for executing that command as described below.
+#
+# If :run_method is :sudo (or :use_sudo is true), this executes the given command
+# via +sudo+. Otherwise is uses +run+. If :as is given as a key, it will be
+# passed as the user to sudo as, if using sudo. If the :as key is not given,
+# it will default to whatever the value of the :admin_runner variable is,
+# which (by default) is unset.
+#
+# THUS, if you want to try to run something via sudo, and what to use the
+# root user, you'd just to try_sudo('something'). If you wanted to try_sudo as
+# someone else, you'd just do try_sudo('something', :as => "bob"). If you
+# always wanted sudo to run as a particular user, you could do 
+# set(:admin_runner, "bob").
+def try_sudo(*args)
+  options = args.last.is_a?(Hash) ? args.pop : {}
+  command = args.shift
+  raise ArgumentError, "too many arguments" if args.any?
+
+  as = options.fetch(:as, fetch(:admin_runner, nil))
+  via = fetch(:run_method, :sudo)
+  if command
+    invoke_command(command, :via => via, :as => as)
+  elsif via == :sudo
+    sudo(:as => as)
+  else
+    ""
+  end
+end
+
+# Same as sudo, but tries sudo with :as set to the value of the :runner
+# variable (which defaults to "app").
+def try_runner(*args)
+  options = args.last.is_a?(Hash) ? args.pop : {}
+  args << options.merge(:as => fetch(:runner, "app"))
+  try_sudo(*args)
+end
+
 # =========================================================================
 # These are the tasks that are available to help with deploying web apps,
 # and specifically, Rails applications. You can have cap give you a summary
@@ -110,19 +166,19 @@ namespace :deploy do
   desc <<-DESC
     Prepares one or more servers for deployment. Before you can use any \
     of the Capistrano deployment tasks with your project, you will need to \
-    make sure all of your servers have been prepared with `cap setup'. When \
+    make sure all of your servers have been prepared with `cap deploy:setup'. When \
     you add a new server to your cluster, you can easily run the setup task \
     on just that server by specifying the HOSTS environment variable:
 
-      $ cap HOSTS=new.server.com setup
+      $ cap HOSTS=new.server.com deploy:setup
 
     It is safe to run this task on servers that have already been set up; it \
     will not destroy any deployed revisions or data.
   DESC
   task :setup, :except => { :no_release => true } do
     dirs = [deploy_to, releases_path, shared_path]
-    dirs += %w(system log pids).map { |d| File.join(shared_path, d) }
-    run "umask 02 && mkdir -p #{dirs.join(' ')}"
+    dirs += shared_children.map { |d| File.join(shared_path, d) }
+    run "#{try_sudo} mkdir -p #{dirs.join(' ')} && #{try_sudo} chmod g+w #{dirs.join(' ')}"
   end
 
   desc <<-DESC
@@ -169,7 +225,9 @@ namespace :deploy do
     symlinks to the shared directory for the log, system, and tmp/pids \
     directories, and will lastly touch all assets in public/images, \
     public/stylesheets, and public/javascripts so that the times are \
-    consistent (so that asset timestamping works).
+    consistent (so that asset timestamping works).  This touch process \
+    is only carried out if the :normalize_asset_timestamps variable is \
+    set to true, which is the default.
   DESC
   task :finalize_update, :except => { :no_release => true } do
     run "chmod -R g+w #{latest_release}" if fetch(:group_writable, true)
@@ -185,9 +243,11 @@ namespace :deploy do
       ln -s #{shared_path}/pids #{latest_release}/tmp/pids
     CMD
 
-    stamp = Time.now.utc.strftime("%Y%m%d%H%M.%S")
-    asset_paths = %w(images stylesheets javascripts).map { |p| "#{latest_release}/public/#{p}" }.join(" ")
-    run "find #{asset_paths} -exec touch -t #{stamp} {} ';'; true", :env => { "TZ" => "UTC" }
+    if fetch(:normalize_asset_timestamps, true)
+      stamp = Time.now.utc.strftime("%Y%m%d%H%M.%S")
+      asset_paths = %w(images stylesheets javascripts).map { |p| "#{latest_release}/public/#{p}" }.join(" ")
+      run "find #{asset_paths} -exec touch -t #{stamp} {} ';'; true", :env => { "TZ" => "UTC" }
+    end
   end
 
   desc <<-DESC
@@ -200,7 +260,14 @@ namespace :deploy do
     except `restart').
   DESC
   task :symlink, :except => { :no_release => true } do
-    on_rollback { run "rm -f #{current_path}; ln -s #{previous_release} #{current_path}; true" }
+    on_rollback do
+      if previous_release
+        run "rm -f #{current_path}; ln -s #{previous_release} #{current_path}; true"
+      else
+        logger.important "no previous release to rollback to, rollback of symlink skipped"
+      end
+    end
+
     run "rm -f #{current_path} && ln -s #{latest_release} #{current_path}"
   end
 
@@ -214,30 +281,29 @@ namespace :deploy do
     To use this task, specify the files and directories you want to copy as a \
     comma-delimited list in the FILES environment variable. All directories \
     will be processed recursively, with all files being pushed to the \
-    deployment servers. Any file or directory starting with a '.' character \
-    will be ignored.
+    deployment servers.
 
       $ cap deploy:upload FILES=templates,controller.rb
+
+    Dir globs are also supported:
+
+      $ cap deploy:upload FILES='config/apache/*.conf'
   DESC
   task :upload, :except => { :no_release => true } do
-    files = (ENV["FILES"] || "").
-      split(",").
-      map { |f| f.strip!; File.directory?(f) ? Dir["#{f}/**/*"] : f }.
-      flatten.
-      reject { |f| File.directory?(f) || File.basename(f)[0] == ?. }
+    files = (ENV["FILES"] || "").split(",").map { |f| Dir[f.strip] }.flatten
+    abort "Please specify at least one file or directory to update (via the FILES environment variable)" if files.empty?
 
-    abort "Please specify at least one file to update (via the FILES environment variable)" if files.empty?
-
-    files.each do |file|
-      content = File.open(file, "rb") { |f| f.read }
-      put content, File.join(current_path, file)
-    end
+    files.each { |file| top.upload(file, File.join(current_path, file)) }
   end
 
   desc <<-DESC
     Restarts your application. This works by calling the script/process/reaper \
     script under the current path.
     
+    If you are deploying a Rails 2.3.x application, you will need to install 
+    these http://github.com/rails/irs_process_scripts (more info about why
+    on that page.)
+    
     By default, this will be invoked via sudo as the `app' user. If \
     you wish to run it as a different user, set the :runner variable to \
     that user. If you are in an environment where you can't use sudo, set \
@@ -246,33 +312,54 @@ namespace :deploy do
       set :use_sudo, false
   DESC
   task :restart, :roles => :app, :except => { :no_release => true } do
-    as = fetch(:runner, "app")
-    via = fetch(:run_method, :sudo)
-    invoke_command "#{current_path}/script/process/reaper", :via => via, :as => as
+    warn "[DEPRECATED] `deploy:restart` is going to be changed to Passenger mod_rails' method after 2.5.9 - see http://is.gd/2BPeA"
+    try_runner "#{current_path}/script/process/reaper"
   end
 
-  desc <<-DESC
-    Rolls back to the previously deployed version. The `current' symlink will \
-    be updated to point at the previously deployed version, and then the \
-    current release will be removed from the servers. You'll generally want \
-    to call `rollback' instead, as it performs a `restart' as well.
-  DESC
-  task :rollback_code, :except => { :no_release => true } do
-    if releases.length < 2
-      abort "could not rollback the code because there is no prior release"
-    else
-      run "rm #{current_path}; ln -s #{previous_release} #{current_path} && rm -rf #{current_release}"
+  namespace :rollback do
+    desc <<-DESC
+      [internal] Points the current symlink at the previous revision.
+      This is called by the rollback sequence, and should rarely (if
+      ever) need to be called directly.
+    DESC
+    task :revision, :except => { :no_release => true } do
+      if previous_release
+        run "rm #{current_path}; ln -s #{previous_release} #{current_path}"
+      else
+        abort "could not rollback the code because there is no prior release"
+      end
     end
-  end
 
-  desc <<-DESC
-    Rolls back to a previous version and restarts. This is handy if you ever \
-    discover that you've deployed a lemon; `cap rollback' and you're right \
-    back where you were, on the previously deployed version.
-  DESC
-  task :rollback do
-    rollback_code
-    restart
+    desc <<-DESC
+      [internal] Removes the most recently deployed release.
+      This is called by the rollback sequence, and should rarely
+      (if ever) need to be called directly.
+    DESC
+    task :cleanup, :except => { :no_release => true } do
+      run "if [ `readlink #{current_path}` != #{current_release} ]; then rm -rf #{current_release}; fi"
+    end
+
+    desc <<-DESC
+      Rolls back to the previously deployed version. The `current' symlink will \
+      be updated to point at the previously deployed version, and then the \
+      current release will be removed from the servers. You'll generally want \
+      to call `rollback' instead, as it performs a `restart' as well.
+    DESC
+    task :code, :except => { :no_release => true } do
+      revision
+      cleanup
+    end
+
+    desc <<-DESC
+      Rolls back to a previous version and restarts. This is handy if you ever \
+      discover that you've deployed a lemon; `cap rollback' and you're right \
+      back where you were, on the previously deployed version.
+    DESC
+    task :default do
+      revision
+      restart
+      cleanup
+    end
   end
 
   desc <<-DESC
@@ -337,7 +424,7 @@ namespace :deploy do
       directories = (releases - releases.last(count)).map { |release|
         File.join(releases_path, release) }.join(" ")
 
-      invoke_command "rm -rf #{directories}", :via => run_method
+      try_sudo "rm -rf #{directories}"
     end
   end
 
@@ -406,9 +493,8 @@ namespace :deploy do
     the :use_sudo variable to false.
   DESC
   task :start, :roles => :app do
-    as = fetch(:runner, "app")
-    via = fetch(:run_method, :sudo)
-    invoke_command "sh -c 'cd #{current_path} && nohup script/spin'", :via => via, :as => as
+    warn "[DEPRECATED] `deploy:start` is going to be removed after 2.5.9 - see http://is.gd/2BPeA"
+    run "cd #{current_path} && #{try_runner} nohup script/spin"
   end
 
   desc <<-DESC
@@ -423,11 +509,9 @@ namespace :deploy do
     the :use_sudo variable to false.
   DESC
   task :stop, :roles => :app do
-    as = fetch(:runner, "app")
-    via = fetch(:run_method, :sudo)
-
-    invoke_command "#{current_path}/script/process/reaper -a kill -r dispatch.spawner.pid", :via => via, :as => as
-    invoke_command "#{current_path}/script/process/reaper -a kill", :via => via, :as => as
+    warn "[DEPRECATED] `deploy:start` is going to be removed after 2.5.9 - see http://is.gd/2BPeA"
+    run "if [ -f #{current_path}/tmp/pids/dispatch.spawner.pid ]; then #{try_runner} #{current_path}/script/process/reaper -a kill -r dispatch.spawner.pid; fi"
+    try_runner "#{current_path}/script/process/reaper -a kill"
   end
 
   namespace :pending do
@@ -472,6 +556,19 @@ namespace :deploy do
       require 'erb'
       on_rollback { run "rm #{shared_path}/system/maintenance.html" }
 
+      warn <<-EOHTACCESS
+      
+        # Please add something like this to your site's htaccess to redirect users to the maintenance page.
+        # More Info: http://www.shiftcommathree.com/articles/make-your-rails-maintenance-page-respond-with-a-503
+        
+        ErrorDocument 503 /system/maintenance.html
+        RewriteEngine On
+        RewriteCond %{REQUEST_URI} !\.(css|gif|jpg|png)$
+        RewriteCond %{DOCUMENT_ROOT}/system/maintenance.html -f
+        RewriteCond %{SCRIPT_FILENAME} !maintenance.html
+        RewriteRule ^.*$  -  [redirect=503,last]
+      EOHTACCESS
+
       reason = ENV['REASON']
       deadline = ENV['UNTIL']
 
